@@ -26,7 +26,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: enable_pps.c,v 1.34 1999/10/10 14:07:07 windl Exp $
+ * $Id: enable_pps.c,v 1.35 1999/10/20 18:55:53 windl Exp $
  */
 #include	<unistd.h>
 #include	<stdlib.h>
@@ -84,7 +84,7 @@
 #define	BITS_OFF(v, b)			((v) & ~(b))
 #define	BITS_ON(v, b)			((v) | (b))
 
-static	const char	id[] = "$Id: enable_pps.c,v 1.34 1999/10/10 14:07:07 windl Exp $";
+static	const char	id[] = "$Id: enable_pps.c,v 1.35 1999/10/20 18:55:53 windl Exp $";
 
 #define	BIT(n)	(1UL << (n))
 #define OPT_MONITOR		BIT(0)	/* perform monitor loop */
@@ -96,6 +96,7 @@ static	const char	id[] = "$Id: enable_pps.c,v 1.34 1999/10/10 14:07:07 windl Exp
 #define OPT_JITTER_LOOP		BIT(6)	/* monitor jitter only */
 #define OPT_MOD_MICRO		BIT(7)	/* enforce MOD_MICRO */
 #define OPT_MOD_NANO		BIT(8)	/* enforce MOD_NANO */
+#define OPT_JITTER_SILENT	BIT(9)	/* only output errors */
 
 static	unsigned long	options = OPT_MONITOR | OPT_RESET;
 
@@ -206,18 +207,14 @@ static	int	pps_status(const char *tag, const int show_event,
 
 	if ( show_event && (cip->have_assert || cip->have_clear) )
 	{
-		long	sec = cip->sec;
+		long	dsec = cip->delta.tv_sec;
 
-		if ( cip->nsec < -500000000 )
-			--sec;
-		else if ( cip->nsec > 500000000 )
-			++sec;
-		fprintf(stderr, "%spulse(%c:%c) %lu at 0.%09ld",
-			sep, cip->have_assert + '0', cip->have_clear + '0',
-			cip->seq, cip->nsec), sep = ", ";
-		if ( sec != cip->dseq )
+		fprintf(stderr, "%spulse(%4lu:%4lu) at 0.%09ld",
+			sep, cip->info.assert_sequence,
+			cip->info.clear_sequence, cip->nsec), sep = ", ";
+		if ( dsec != cip->dseq )
 			fprintf(stderr, "%s%ld events in %ld seconds",
-				sep, cip->dseq, sec), sep = ", ";
+				sep, cip->dseq, dsec), sep = ", ";
 		result = 0;
 	}
 	switch ( cip->state )
@@ -251,6 +248,7 @@ static	int	update_clock_info(struct clock_info *cip)
 {
 	static struct clock_info	last;
 	static struct timespec	sleeper = {0, 0};
+	pps_seq_t	ds;
 	int	result = -1;
 
 	cip->tx.modes = 0;
@@ -261,27 +259,26 @@ static	int	update_clock_info(struct clock_info *cip)
 		perror("time_pps_fetch()");
 		memset(&cip->info, 0, sizeof(cip->info));
 	}
-	
-	cip->have_assert = cip->info.assert_sequence != last.info.assert_sequence;
-	cip->have_clear = cip->info.clear_sequence != last.info.clear_sequence;
-	if ( cip->have_assert )
-	{
+
+	cip->dseq = cip->have_assert = cip->have_clear = 0;
+	if ( (ds = cip->info.assert_sequence - last.info.assert_sequence)
+	     != 0 ) {
+		cip->have_assert = 1;
+		cip->dseq += ds;
 		cip->sec = cip->info.assert_timestamp.tv_sec;
-		cip->dsec = cip->sec - last.sec;
 		cip->nsec = cip->info.assert_timestamp.tv_nsec;
-		cip->dnsec = cip->nsec - last.nsec;
 		cip->seq = cip->info.assert_sequence;
-		cip->dseq = cip->seq - last.seq;
 	}
-	if ( cip->have_clear )
-	{
+	if ( (ds = cip->info.clear_sequence - last.info.clear_sequence)
+	     != 0 ) {
+		cip->have_clear = 1;
+		cip->dseq += ds;
 		cip->sec = cip->info.clear_timestamp.tv_sec;
-		cip->dsec = cip->sec - last.sec;
 		cip->nsec = cip->info.clear_timestamp.tv_nsec;
-		cip->dnsec = cip->nsec - last.nsec;
 		cip->seq = cip->info.clear_sequence;
-		cip->dseq = cip->seq - last.seq;
 	}
+	cip->dsec = cip->sec - last.sec;
+	cip->dnsec = cip->nsec - last.nsec;
 	if ( cip->dseq )
 	{
 		cip->nano_offset = cip->nsec;
@@ -620,17 +617,23 @@ static	void	monitor_jitter(struct clock_info *cip)
 		if ( cip->seq == last.seq )
 			continue;
 		pps_status("monitor_jitter(): ", 0, cip);
-		printf("event(%4lu:%4lu) jitter %9ld ns, offset %9ld ns",
-		       cip->info.assert_sequence, cip->info.clear_sequence,
-		       cip->jitter, cip->nano_offset);
-		if ( cip->jitter > 1000000 || cip->jitter < -1000000 )
-		{
-			printf("\t%ld:%ld %ld:%ld %ld:%ld",
-			       last.sec, last.nsec, cip->sec, cip->nsec,
-			       cip->delta.tv_sec, cip->delta.tv_nsec);
+		if ( (options & OPT_JITTER_SILENT) == 0 ||
+		     (cip->tx.status & ALL_ERRORS) != 0 ) {
+			printf("event(%4lu:%4lu) jitter %9ld ns, offset %9ld ns",
+			       cip->info.assert_sequence,
+			       cip->info.clear_sequence,
+			       cip->jitter, cip->nano_offset);
+			if ( (cip->tx.status & STA_PPSJITTER) != 0 ||
+			     cip->jitter > 1000000 || cip->jitter < -1000000 )
+			{
+				printf("\t%ld:%ld %ld:%ld %ld:%ld",
+				       last.sec, last.nsec,
+				       cip->sec, cip->nsec,
+				       cip->delta.tv_sec, cip->delta.tv_nsec);
+			}
+			printf("\n");
 		}
 		last = *cip;
-		printf("\n");
 		fflush(stdout);
 	}
 }
@@ -644,6 +647,7 @@ static	void	usage(void)
 		"\t-EX\tEcho - set PPS API echo for `X' events (a|c)\n"
 		"\t-e\texit - do not enter monitoring loop\n"
 		"\t-F\tdon't adjust frequency in monitoring loop\n"
+		"\t-J\tmonitor jitter for PPS errors only\n"
 		"\t-j\tmonitor jitter only\n"
 		"\t-k\tkeep kernel flags (when entering monitoring loop)\n"
 #if defined(MOD_MICRO) && defined(MOD_NANO)
@@ -685,7 +689,7 @@ int	main(int argc, char *argv[])
 	uname(&un);
 	printf(". Now running %s %s)\n", un.sysname, un.release);
 	parm.mode = PPS_TSFMT_TSPEC;	/* set default flags */
-	while ( (ch = getopt(argc, argv, "CE:eFjkMNOo:St:")) != -1 )
+	while ( (ch = getopt(argc, argv, "CE:eFJjkMNOo:St:")) != -1 )
 	{
 		switch ( ch )
 		{
@@ -714,6 +718,9 @@ int	main(int argc, char *argv[])
 		case 'F':	/* don't adjust frequency */
 			options = BITS_ON(options, OPT_NO_FREQ);
 			break;
+		case 'J':	/* like '-j', print only error conditions */
+			options = BITS_ON(options, OPT_JITTER_SILENT);
+			/* fall through */
 		case 'j':	/* monitor jitter only */
 			options = BITS_ON(options, OPT_JITTER_LOOP);
 			break;
